 /*
 * file      : callbacks.c
 * project   : BaseBuild
 * with      : Gtk-2
 *
 * copyright : (C) 2010 by Claude Bulin
 *
 * BaseBuild - Creation d'une base de programmation en langage C de type GNU avec les autotools
 * GNU General Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * OLD ADRESS:
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * NEW ADRESS:
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gstdio.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

#include "../libbasebuild/libbasebuild.h"
#include "global.h"
#include "configuser.h"
#include "fileselect.h"
#include "callbacks.h"
#include "build.h"
#include "win_info.h"
#include "win_about.h"



VAR_CALLBACKS VarCallbacks;



// -RECUPERATION DES DATAS
// -SAUVEGARDE DES DATAS
// 
void remove_memory_apply (void)
{
	g_print ("\n--------------------------------------------------\n");
	PRINT_FUNC_LF();
	g_print ("--------------------------------------------------\n");
	
	g_print ("Recuperation position et taille de la fenetre\n");
	gdk_window_get_root_origin (WindMain->window, &Config.WinPos_X, &Config.WinPos_Y);
	gdk_window_get_size (WindMain->window, &Config.WinWidth, &Config.WinHeight);	
	
	g_print ("Sauvegarde de la configuration\n");
	config_save ();
	
	g_print ("config_remove ()\n");
	config_remove ();
	g_print ("remove_combo ()\n");
	libcombo_remove ();
	g_print ("build_remove ()\n");
	build_remove ();
	
	g_print ("\n");
}
// QUIT WITH delete_event
//
gboolean on_window_main_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	remove_memory_apply ();
	gtk_main_quit ();
	return FALSE;
}
// QUIT WITH destroy_event
//
gboolean on_window_main_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	remove_memory_apply ();
	gtk_main_quit ();
	return FALSE;
}
// QUIT WITH menu
//
void on_imagemenuitem_quit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	remove_memory_apply ();
	gtk_main_quit ();
}
// MENU ABOUT
//
void on_imagemenuitem_about_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	WinAbout_open ();
}
// INIT PATH PROJET
//
void on_button_path_projet_realize (GtkWidget *widget)
{
	VarCallbacks.Adr_button_path = GTK_BUTTON (widget);
	gtk_button_set_use_underline (GTK_BUTTON (widget), FALSE);
	gtk_button_set_label (GTK_BUTTON (widget), Config.Path);
}
// TEST L ACCESSIBLITE AU DOSSIER
//
gboolean Callbacks_test_write (gchar *path)
{
	gboolean  BoolWriteOk = FALSE;
	FILE     *fp = NULL;
	gchar    *str = NULL;
	
	if (path[ 0 ] == '/' && path[ 1 ] != '\0') {
		str = g_strdup_printf ("%s/basebuild_tst_write_dummy.txt", path);
		if (NULL != (fp = fopen (str, "w"))) {
			BoolWriteOk = TRUE;
			fclose (fp);
			g_unlink (str);
		}
		g_free (str);
		str = NULL;
	}
	
	// TODO: FENETRE D ALERTE POUR L UTILISATEUR
	if (BoolWriteOk == FALSE) {
		wininfo_create (
			_("Erreur choix lieu de stockage !"),
			_("   Le lieu de stockage temporaire doit pouvoir     "),
			  "\n",
			_("   etre accessible en ecriture."),
			  "\n\n",
			_("   Veuillez recommencer."),
			  "\n",
			  "");
	}
	
	return (BoolWriteOk);
}
// CHANGEMENT DE PATH DU PROJET
//
void file_select_new_path_destination (gchar *path)
{
	if (TRUE == Callbacks_test_write (path)) {
		g_free (Config.Path);
		Config.Path = NULL;
		Config.Path = g_strdup (path);
		gtk_button_set_use_underline (GTK_BUTTON (VarCallbacks.Adr_button_path), FALSE);
		gtk_button_set_label (GTK_BUTTON (VarCallbacks.Adr_button_path), Config.Path);
	}
}
// CHANGEMENT DE PATH PROJET
// 
void on_button_path_projet_clicked (GtkButton *button, gpointer user_data)
{
	fileselect ("Choix du dossier pour le projet", TRUE, file_select_new_path_destination, Config.Path);
}
// INIT NOM PROJET
//
void on_entry_name_projet_realize (GtkWidget *widget, gpointer user_data)
{
	if ('\0' == *Config.NameProject) {
		g_free (Config.NameProject);
		Config.NameProject = NULL;
		Config.NameProject = g_strdup ("NameProject");
	}
	gtk_entry_set_text (GTK_ENTRY (widget), Config.NameProject);
	VarCallbacks.Adr_name_project = widget;
}
// MODIFICATION NOM DU PROJET
//
void on_entry_name_projet_changed (GtkEditable *editable, gpointer user_data)
{
	if (NULL != VarCallbacks.Adr_name_project) {
		g_free (Config.NameProject);
		Config.NameProject = NULL;
		Config.NameProject = g_strdup( gtk_entry_get_text (GTK_ENTRY (VarCallbacks.Adr_name_project)));
		
		// DE ... ACTIVATION DU BOUTON APPLIQUER
		gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("button_appliquer")),
			('\0' == *gtk_entry_get_text (GTK_ENTRY (VarCallbacks.Adr_name_project))) ? FALSE : TRUE);
	}
	// button_appliquer
}
// INIT SPINBUTTON VERSION A B C
//
void on_spinbutton_version_A_realize (GtkWidget *widget, gpointer user_data)
{
	VarCallbacks.Adr_spinbutton_version_A = widget;
}
void on_spinbutton_version_B_realize (GtkWidget *widget, gpointer user_data)
{
	VarCallbacks.Adr_spinbutton_version_B = widget;
}
void on_spinbutton_version_C_realize (GtkWidget *widget, gpointer user_data)
{
	VarCallbacks.Adr_spinbutton_version_C = widget;
}
// CHANGEMENT NUMERO DE VERSION
//
void on_spinbutton_version_changed (GtkSpinButton *spinbutton, gpointer user_data)
{
	/*
	if (NULL != VarCallbacks.Adr_spinbutton_version_A &&
	    NULL != VarCallbacks.Adr_spinbutton_version_B &&
	    NULL != VarCallbacks.Adr_spinbutton_version_C) {
		// GENERATION FICHIERS ET AFFICHAGE
		build_view_struct ();
	}
	*/
}
// INIT TEWTVIEW
//
void on_textview_voir_fichiers_realize (GtkWidget *widget, gpointer user_data)
{
	VarCallbacks.Adr_textview = widget;
}
// CONFIRME CREATE BUILDER
// 
void on_button_appliquer_clicked (GtkButton *button, gpointer user_data)
{
	VarCallbacks.BoolCreateBuild = TRUE;
	build_clicked ();
}
// CHOICE WITH CLICK ON REP
// 
void on_checkbutton_dossier_clicked (GtkButton *button, gpointer user_data)
{
	if (button == GTK_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu"))) {
		if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu")))) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_pixmaps")), TRUE);
			// gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("combobox_categories_menu")), TRUE);
		}
	}
	else if (button == GTK_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_pixmaps"))) {
		if (FALSE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_pixmaps"))) &&
			TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu")))) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu")), FALSE);
			// gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("combobox_categories_menu")), FALSE);
		}
	}
	
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("combobox_categories_menu")), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu"))));
}
// CHOICE WITH CLICK ON LIB
//
void on_checkbutton_lib_clicked (GtkButton *button, gpointer user_data)
{

	// gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("frame_paquet_glade")),
	// 		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_lib_gtk"))));
}
// INIT COMBOBOX CATEGORIES
//
void on_combobox_categories_menu_realize (GtkWidget *widget, gpointer user_data)
{
	VarCallbacks.Adr_combobox_categories = GTK_COMBO_BOX (widget);

	libcombo_alloc (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories));
	
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "AudioVideo");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Audio");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Video");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Development");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Education");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Game");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Graphics");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Network");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Office");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Settings");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "System");
	gtk_combo_box_append_text (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), "Utility");
	
	gtk_combo_box_set_active (GTK_COMBO_BOX (VarCallbacks.Adr_combobox_categories), 3);
}
// RETOURNE LA CATEGORIE ACTIVE
//
gchar *Callbacks_get_entry_categories_item (void)
{
	*VarCallbacks.StrCategoriesEntry = '\0';
	g_sprintf (&VarCallbacks.StrCategoriesEntry[0], "Categories=Application;%s\n", gtk_combo_box_get_active_text (GTK_COMBO_BOX(VarCallbacks.Adr_combobox_categories)));
	return (VarCallbacks.StrCategoriesEntry);
}







